<?php

namespace Drupal\json_users_import\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\user\Entity\User;

/*
 * Contoller for Json User Import
 */

class JsonUsersImportController extends ControllerBase {

  /**
   * Returns user data if email id is exists.
   *
   * @param string|$email
   *   Email id from the currently uploaded json data.
   *
   * @return array
   *   User Id and Error message.
   */
  public function isEmailExist($email) {
    $userId = \Drupal::entityQuery('user')->condition('mail', $email)->execute();
    $userId = array_shift($userId);
    if($userId > 0) {
      return array ('id' => $userId, 'returnMsg' => $email.' is already exists!');
    }
  }

  /**
   * Returns user data if username is exists.
   *
   * @param string|$username
   *   Username from the currently uploaded json data.
   *
   * @return array
   *   User Id and Error message.
   */
  public function isUserNameExist($username) {
    $userId = \Drupal::entityQuery('user')->condition('name', $username)->range(0, 1)
        ->execute();
    $userId = array_shift($userId);
    if($userId > 0) {
      return array ('id' => $userId, 'returnMsg' => $username.' is already exists!');
    }
  }

  /**
   * Returns array if username is valid.
   *
   * @param string|$username
   *   Username from the currently uploaded json data.
   *
   * @return array
   *   Return FALSE, if username is valid and Error message.
   *   Otherwise return TRUE.
   */
  public function isUserNameValid($username) {
    $valid = TRUE;
    $returnMsg = '';
    if (substr($username, 0, 1) == ' ') {
      $valid = FALSE;
      $returnMsg = $username .' cannot begin with a space.';
    } else if (substr($username, -1) == ' ') {
      $valid = FALSE;
      $returnMsg = $username .' cannot end with a space.';
    } else if (strpos($username, '  ') !== FALSE) {
      $valid = FALSE;
      $returnMsg = $username .' cannot contain multiple spaces.';
    } else if (preg_match('/[^\x{80}-\x{F7} a-z0-9@+_.\'-]/i', $username)) {
      $valid = FALSE;
      $returnMsg = $username .' contains an illegal character.';
    } else if (mb_strlen($username) > 60 ) {
      $valid = FALSE;
      $returnMsg = $username .' is too long, it must be 60 characters or less.';
    }
    return array ('valid' => $valid, 'returnMsg' => $returnMsg);
  }

  /**
   * Create user data if user not exists.
   *
   * @param array|$value
   *   An array of user data from the currently uploaded json data.
   *
   * @param string|$emailKey
   *   Email key from the currently uploaded json data.
   *
   * @param string|$usernameKey
   *   Username key from the currently uploaded json data.
   *
   * @param array|$jsonKey
   *   An array of json key from the user import configuration.
   *
   * @return string
   *   Status of the created user record.
   */
  public function createUser($value, $emailKey, $usernameKey, $jsonKey) {
    $language = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $statusMsg = '';
    $random = new \Drupal\Component\Utility\Random();
    $password = $random->name(7);
    $user = User::create();
    $user->setEmail($value[$emailKey]);
    $user->setUsername($value[$usernameKey]);
    $user->setPassword($password);
    $user->enforceIsNew();
    $user->set('init', $value[$emailKey]);
    $user->set('langcode', $language);
    $user->set('preferred_langcode', $language);
    $user->set('preferred_admin_langcode', $language);
    $user->activate();
    $ignoreFields = array("email", "name", "password");

    foreach($jsonKey as $fieldName => $fieldValue) {
      if (!in_array($fieldName, $ignoreFields)) {
        $user->set($fieldName,$value[$fieldValue]);
      }
    }
    // Save user account.
    $result = $user->save();
    if ($user->isActive()) {
      $userId = $user->id();
      $statusMsg = 'User '. $value[$usernameKey] .' is created successfully!';
      \Drupal::logger('json_users_import')->notice($statusMsg);
    } else {
      $statusMsg = 'User '. $value[$usernameKey] .' is not created!';
      \Drupal::logger('json_users_import')->error($statusMsg);
    }
    return array ('statusMsg' => $statusMsg, 'userId' => $userId);
  }
} 
